import 'dart:async';
import 'dart:convert';
import 'package:http/http.dart';
import 'package:reflutter/reflutter.dart';
import 'requests/requests.dart';
import 'responses/responses.dart';

export 'models/models.dart';
export 'responses/responses.dart';
export 'requests/requests.dart';

part 'catalog.api.dart';

@ReflutterHttp(name: 'SquareCatalog')
abstract class _SquareCatalogEndpoint {
  @Post('/v2/catalog/batch-delete')
  Future<ReflutterResponse<BatchDeleteResponse>> batchDelete();

  @Post('/v2/catalog/batch-retrieve')
  Future<ReflutterResponse<BatchRetrieveResponse>> batchRetrieve(
    @Body() BatchRetrieveBody body
  );

  @Post('/v2/catalog/batch-upsert')
  Future<ReflutterResponse<BatchUpsertResponse>> batchUpsert(
    @Body() BatchUpsertBody body
  );

  @Get('/v2/catalog/info')
  Future<ReflutterResponse<CatalogInfoResponse>> catalogInfo();

  @Delete('/v2/catalog/object/{objectId}')
  Future<ReflutterResponse<DeleteCatalogResponse>> deleteCatalog(
    @Param() String objectId
  );

  @Get('/v2/catalog/list')
  Future<ReflutterResponse<ListCatalogResponse>> listCatalog(
    @QueryParam() String cursor, 
    @QueryParam() String types
  );

  @Get('/v2/catalog/object')
  Future<ReflutterResponse<RetrieveCatalogResponse>> retrieveCatalog(
    String objectId, 
    { @QueryParam('include_related_objects') bool includeRelatedObjects }
  );

  @Post('/v2/catalog/search')
  Future<ReflutterResponse<SearchCatalogsResponse>> searchCatalogs(
    @Body() SearchCatalogsRequest req
  );

  @Post('/v2/catalog/update-item-modifier-lists')
  Future<ReflutterResponse<UpdateItemResponse>> updateItemModifiers(
    @Body() UpdateItemModifiersRequest req
  );

  @Post('/v2/catalog/update-item-taxes')
  Future<ReflutterResponse<UpdateItemResponse>> updateItemTaxes(
    @Body() UpdateItemTaxesRequest req
  );

  @Get('/v2/catalog/object')
  Future<ReflutterResponse<UpsertCatalogResponse>> upsertCatalog(
    @Body() UpsertCatalogRequest req
  );
}