import 'package:json_annotation/json_annotation.dart';

part 'item.g.dart';

@JsonSerializable()
class CatalogItem {
  final String name;
  final String description;
  final String abbreviation;
  @JsonValue('label_color')
  final String labelColor;
  @JsonValue('available_online')
  final bool availableOnline;
  @JsonValue('available_for_pickup')
  final bool availableForPickup;
  @JsonValue('available_electronically')
  final bool availableElectronically;
  @JsonValue('category_id')
  final String categoryId;
  @JsonValue('tax_ids')
  final List<String> taxIds;

  CatalogItem({this.name, this.description, this.abbreviation, this.labelColor, 
               this.availableOnline, this.availableElectronically, this.availableForPickup, 
               this.categoryId, this.taxIds});

  factory CatalogItem.fromJson(Map<String, dynamic> json) => _$CatalogItemFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogItemToJson(this);
}

@JsonSerializable()
class CatalogItemModifierListInfo {
  @JsonValue('modifier_list_id')
  final String modifierListId;

  @JsonValue('modifier_overrides')
  final List<CatalogModifierOverride> modifierOverrides;
  
  @JsonValue('min_selected_modifiers')
  final int minSelectedModifiers;

  @JsonValue('max_selected_modifiers')
  final int maxSelectedModifiers;

  final bool enabled;

  CatalogItemModifierListInfo({this.modifierListId, this.modifierOverrides, this.minSelectedModifiers, this.maxSelectedModifiers, this.enabled});

  factory CatalogItemModifierListInfo.fromJson(Map<String, dynamic> json) => _$CatalogItemModifierListInfoFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogItemModifierListInfoToJson(this);
}

@JsonSerializable()
class CatalogModifierOverride {
  @JsonValue('modifier_id')
  final String modifierId;

  @JsonValue('on_by_default')
  final bool onByDefault;

  CatalogModifierOverride({this.modifierId, this.onByDefault});

  factory CatalogModifierOverride.fromJson(Map<String, dynamic> json) => _$CatalogModifierOverrideFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogModifierOverrideToJson(this);
}