import 'package:json_annotation/json_annotation.dart';

part 'query.g.dart';

@JsonSerializable()
class CatalogQuery {
  @JsonValue('sorted_attribute_query')
  CatalogQuerySortedAttribute sortedAttributeQuery;
  @JsonValue('exact_query')
  CatalogQueryExact exactQuery;
  @JsonValue('prefix_query')
  CatalogQueryPrefix prefixQuery;
  @JsonValue('range_query')
  CatalogQueryRange rangeQuery;
  @JsonValue('text_query')
  CatalogQueryText textQuery;
  @JsonValue('items_for_tax_query')
  CatalogQueryItemsForTax itemsForTaxQuery;
  @JsonValue('items_for_modifier_list_query')
  CatalogQueryItemsForModifierList itemsForModifierListQuery;

  CatalogQuery({this.sortedAttributeQuery, this.exactQuery, this.prefixQuery, 
                this.rangeQuery, this.textQuery, this.itemsForTaxQuery, 
                this.itemsForModifierListQuery});

  factory CatalogQuery.fromJson(Map<String, dynamic> json) => _$CatalogQueryFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogQueryToJson(this);
}

@JsonSerializable()
class CatalogQuerySortedAttribute {
  @JsonValue('attribute_name')
  String attributeName;
  @JsonValue('initial_attribute_value')
  String initialAttributeValue;
  @JsonValue('sort_order')
  String sortOrder;

  CatalogQuerySortedAttribute({this.attributeName, this.initialAttributeValue, this.sortOrder});

  factory CatalogQuerySortedAttribute.fromJson(Map<String, dynamic> json) => _$CatalogQuerySortedAttributeFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogQuerySortedAttributeToJson(this);
}

@JsonSerializable()
class CatalogQueryExact {
  @JsonValue('attribute_name')
  String attributeName;
  @JsonValue('attribute_value')
  String attributeValue;

  CatalogQueryExact({this.attributeName, this.attributeValue});

  factory CatalogQueryExact.fromJson(Map<String, dynamic> json) => _$CatalogQueryExactFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogQueryExactToJson(this);
}

@JsonSerializable()
class CatalogQueryPrefix {
  @JsonValue('attribute_name')
  String attributeName;
  @JsonValue('attribute_prefix')
  String attributePrefix;

  CatalogQueryPrefix({this.attributeName, this.attributePrefix});

  factory CatalogQueryPrefix.fromJson(Map<String, dynamic> json) => _$CatalogQueryPrefixFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogQueryPrefixToJson(this);
}

@JsonSerializable()
class CatalogQueryRange {
  @JsonValue('attribute_name')
  String attributeName;
  @JsonValue('attribute_min_value')
  String attributeMinValue;
  @JsonValue('attribute_max_value')
  String attributeMaxValue;

  CatalogQueryRange({this.attributeName, this.attributeMinValue, this.attributeMaxValue});

  factory CatalogQueryRange.fromJson(Map<String, dynamic> json) => _$CatalogQueryRangeFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogQueryRangeToJson(this);
}

@JsonSerializable()
class CatalogQueryText {
  String name;
  @JsonValue('calculation_phase')
  String calculationPhase;
  @JsonValue('inclusion_type')
  String inclusionType;
  String percentage;
  @JsonValue('applies_to_custom_amounts')
  bool appliesToCustomAmounts;
  bool enabled;

  CatalogQueryText({this.name, this.calculationPhase, this.inclusionType,
                    this.percentage, this.appliesToCustomAmounts, this.enabled});

  factory CatalogQueryText.fromJson(Map<String, dynamic> json) => _$CatalogQueryTextFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogQueryTextToJson(this);
}

@JsonSerializable()
class CatalogQueryItemsForTax {
  @JsonValue('tax_ids')
  List<String> taxIds;

  CatalogQueryItemsForTax({ this.taxIds });

  factory CatalogQueryItemsForTax.fromJson(Map<String, dynamic> json) => _$CatalogQueryItemsForTaxFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogQueryItemsForTaxToJson(this);
}

@JsonSerializable()
class CatalogQueryItemsForModifierList {
  @JsonValue('modifier_list_ids')
  List<String> modifierListIds;

  CatalogQueryItemsForModifierList({ this.modifierListIds });

  factory CatalogQueryItemsForModifierList.fromJson(Map<String, dynamic> json) => _$CatalogQueryItemsForModifierListFromJson(json);
  Map<String, dynamic> toJson() => _$CatalogQueryItemsForModifierListToJson(this);
}