import 'package:json_annotation/json_annotation.dart';
import 'package:dart_square/shared.dart';

part 'create.g.dart';

@JsonSerializable()
class CreateCheckoutRequest {
  @JsonValue('idempotency_key')
  String idempotencyKey;
  OrderRequest order;
  @JsonValue('ask_for_shipping_address')
  bool askForShippingAddress;
  @JsonValue('merchant_support_email')
  String merchantSupportEmail;
  @JsonValue('pre_populate_buyer_email')
  String prePopulateBuyerEmail;
  @JsonValue('pre_populate_shipping_address')
  Address prePopulateShippingAddress;
  @JsonValue('redirect_url')
  String redirectUrl;
  @JsonValue('additional_recipients')
  List<ChargeRequestAdditionalRecipient> additionalRecipients;
  String note;

  CreateCheckoutRequest(this.idempotencyKey, this.order, { this.askForShippingAddress, 
                        this.merchantSupportEmail, this.prePopulateBuyerEmail, 
                        this.prePopulateShippingAddress, this.redirectUrl, this.note, 
                        this.additionalRecipients });

  factory CreateCheckoutRequest.fromJson(Map<String, dynamic> json) => _$CreateCheckoutRequestFromJson(json);
  Map<String, dynamic> toJson() => _$CreateCheckoutRequestToJson(this);
}

@JsonSerializable()
class OrderRequest {
  @JsonValue('idempotency_key')
  String idempotencyKey;
  @JsonValue('reference_id')
  String referenceId;
  @JsonValue('line_items')
  List<RequestLineItem> lineItems;
  List<RequestTax> taxes;
  List<RequestDiscount> discounts;

  OrderRequest({this.idempotencyKey, this.referenceId, this.lineItems, this.taxes, this.discounts });

  factory OrderRequest.fromJson(Map<String, dynamic> json) => _$OrderRequestFromJson(json);
  Map<String, dynamic> toJson() => _$OrderRequestToJson(this);
}

@JsonSerializable()
class RequestDiscount {
  @JsonValue('catalog_object_id')
  String catalogObjectId;
  String name;
  String percentage;
  @JsonValue('amount_money')
  Money amountMoney;

  RequestDiscount({this.catalogObjectId, this.name, this.percentage, this.amountMoney });

  factory RequestDiscount.fromJson(Map<String, dynamic> json) => _$RequestDiscountFromJson(json);
  Map<String, dynamic> toJson() => _$RequestDiscountToJson(this);
}

@JsonSerializable()
class RequestLineItem {
  String name;
  String quantity;
  @JsonValue('base_price_money')
  Money basePriceMoney;
  @JsonValue('variation_name')
  String variationName;
  String note;
  @JsonValue('catalog_object_id')
  String catalogObjectId;
  List<RequestModifier> modifiers;
  List<RequestTax> taxes;
  List<RequestDiscount> discounts;

  RequestLineItem({this.catalogObjectId, this.name, this.quantity, this.basePriceMoney, 
                   this.variationName, this.note, this.modifiers, this.discounts, this.taxes});

  factory RequestLineItem.fromJson(Map<String, dynamic> json) => _$RequestLineItemFromJson(json);
  Map<String, dynamic> toJson() => _$RequestLineItemToJson(this);
}

@JsonSerializable()
class RequestModifier {
  @JsonValue('catalog_object_id')
  String catalogObjectId;
  String name;
  @JsonValue('base_price_money')
  Money basePriceMoney;

  RequestModifier({this.catalogObjectId, this.name, this.basePriceMoney });

  factory RequestModifier.fromJson(Map<String, dynamic> json) => _$RequestModifierFromJson(json);
  Map<String, dynamic> toJson() => _$RequestModifierToJson(this);
}

@JsonSerializable()
class RequestTax {
  @JsonValue('catalog_object_id')
  String catalogObjectId;
  String name;
  String type;
  String percentage;

  RequestTax({this.catalogObjectId, this.name, this.type, this.percentage });

  factory RequestTax.fromJson(Map<String, dynamic> json) => _$RequestTaxFromJson(json);
  Map<String, dynamic> toJson() => _$RequestTaxToJson(this);
}

@JsonSerializable()
class ChargeRequestAdditionalRecipient {
  @JsonValue('location_id')
  String locationId;
  String description;
  @JsonValue('amount_money')
  Money amountMoney;

  ChargeRequestAdditionalRecipient({this.locationId, this.description, this.amountMoney });

  factory ChargeRequestAdditionalRecipient.fromJson(Map<String, dynamic> json) => _$ChargeRequestAdditionalRecipientFromJson(json);
  Map<String, dynamic> toJson() => _$ChargeRequestAdditionalRecipientToJson(this);
}