import 'package:json_annotation/json_annotation.dart';
import 'package:dart_square/shared.dart';

part 'create.g.dart';

@JsonSerializable()
class CreateOrderRequest {
  @JsonValue('idempotency_key')
  final String idempotencyKey;
  @JsonValue('reference_id')
  final String referenceId;
  @JsonValue('line_items')
  final List<CreateOrderRequestLineItem> lineItems;
  final List<CreateOrderRequestTax> taxes;
  final List<CreateOrderRequestDiscount> discounts;
  

  CreateOrderRequest({ this.idempotencyKey, this.referenceId, this.lineItems, this.taxes, this.discounts });

  factory CreateOrderRequest.fromJson(Map<String, dynamic> json) => _$CreateOrderRequestFromJson(json);
  Map<String, dynamic> toJson() => _$CreateOrderRequestToJson(this);
}

@JsonSerializable()
class CreateOrderRequestDiscount {
  @JsonValue('catalog_object_id')
  final String catalogObjectId;
  final String name;
  final String percentage;
  @JsonValue('amount_money')
  final Money amount;
  
  CreateOrderRequestDiscount({ this.catalogObjectId, this.name, this.percentage, this.amount });

  factory CreateOrderRequestDiscount.fromJson(Map<String, dynamic> json) => _$CreateOrderRequestDiscountFromJson(json);
  Map<String, dynamic> toJson() => _$CreateOrderRequestDiscountToJson(this);
}

@JsonSerializable()
class CreateOrderRequestLineItem {
  final String name;
  final String quantity;
  @JsonValue('base_price_money')
  final Money basePrice;
  @JsonValue('variation_name')
  final String variationName;
  final String note;
  @JsonValue('catalog_object_id')
  final String catalogObjectId;
  final List<CreateOrderRequestModifier> modifiers;
  final List<CreateOrderRequestTax> taxes;
  final List<CreateOrderRequestDiscount> discounts;

  CreateOrderRequestLineItem({ this.name, this.quantity, this.basePrice, this.variationName, 
    this.note, this.catalogObjectId, this.modifiers, this.taxes, this.discounts });

  factory CreateOrderRequestLineItem.fromJson(Map<String, dynamic> json) => _$CreateOrderRequestLineItemFromJson(json);
  Map<String, dynamic> toJson() => _$CreateOrderRequestLineItemToJson(this);
}

@JsonSerializable()
class CreateOrderRequestModifier {
  @JsonValue('catalog_object_id')
  final String catalogObjectId;
  final String name;
  @JsonValue('base_price_money')
  final Money basePrice;
  
  CreateOrderRequestModifier({ this.catalogObjectId, this.name, this.basePrice });

  factory CreateOrderRequestModifier.fromJson(Map<String, dynamic> json) => _$CreateOrderRequestModifierFromJson(json);
  Map<String, dynamic> toJson() => _$CreateOrderRequestModifierToJson(this);
}

@JsonSerializable()
class CreateOrderRequestTax {
  @JsonValue('catalog_object_id')
  final String catalogObjectId;
  final String name;
  final String type;
  final String percentage;
  
  CreateOrderRequestTax({ this.catalogObjectId, this.name, this.type, this.percentage });

  factory CreateOrderRequestTax.fromJson(Map<String, dynamic> json) => _$CreateOrderRequestTaxFromJson(json);
  Map<String, dynamic> toJson() => _$CreateOrderRequestTaxToJson(this);
}