import 'package:json_annotation/json_annotation.dart';
import 'money.dart';

part 'order.g.dart';

@JsonSerializable()
class Order {
  final String id;
  @JsonValue('location_id')
  final String locationId;
  @JsonValue('reference_id')
  final String referenceId;
  @JsonValue('line_items')
  final List<OrderLineItem> lineItems;
  @JsonValue('total_money')
  final Money totalMoney;
  @JsonValue('total_tax_money')
  final Money totalTaxMoney;
  @JsonValue('total_discount_money')
  final Money totalDiscountMoney;

  Order({ this.id, this.locationId, this.referenceId, this.lineItems,
          this.totalMoney, this.totalDiscountMoney, this.totalTaxMoney });

  factory Order.fromJson(Map<String, dynamic> json) => _$OrderFromJson(json);
  Map<String, dynamic> toJson() => _$OrderToJson(this);
}

@JsonSerializable()
class OrderLineItem {
  final String name;
  final String quantity;
  final String note;
  @JsonValue('catalog_order_id')
  final String catalogOrderId;
  @JsonValue('variation_name')
  final String variationName;
  final List<LineItemModifiers> modifiers;
  final List<LineItemTax> taxes;
  final List<LineItemDiscount> discounts;
  @JsonValue('base_price_money')
  final Money basePriceMoney;
  @JsonValue('gross_sales_money')
  final Money grossSalesMoney;
  @JsonValue('total_tax_money')
  final Money totalTaxMoney;
  @JsonValue('total_discount_money')
  final Money totalDiscountMoney;
  @JsonValue('total_money')
  final Money totalMoney;

  OrderLineItem({this.name, this.quantity, this.note, this.catalogOrderId, 
                 this.variationName, this.modifiers, this.taxes, this.discounts, 
                 this.basePriceMoney, this.grossSalesMoney, this.totalTaxMoney, 
                 this.totalDiscountMoney, this.totalMoney});

  factory OrderLineItem.fromJson(Map<String, dynamic> json) => _$OrderLineItemFromJson(json);
  Map<String, dynamic> toJson() => _$OrderLineItemToJson(this);
}

@JsonSerializable()
class LineItemModifiers {
  @JsonValue('catalog_object_id')
  final String catalogObjectId;
  final String name;
  @JsonValue('base_price_money')
  final Money basePriceMoney;
  @JsonValue('total_price_money')
  final Money totalPriceMoney;

  LineItemModifiers(this.catalogObjectId, this.name, this.basePriceMoney, this.totalPriceMoney);

  factory LineItemModifiers.fromJson(Map<String, dynamic> json) => _$LineItemModifiersFromJson(json);
  Map<String, dynamic> toJson() => _$LineItemModifiersToJson(this);
}

@JsonSerializable()
class LineItemTax {
  @JsonValue('catalog_object_id')
  final String catalogObjectId;
  final String name;
  final String type;
  final String percentage;
  @JsonValue('applied_money')
  final Money appliedMoney;

  LineItemTax(this.catalogObjectId, this.name, this.type, this.percentage, this.appliedMoney);

  factory LineItemTax.fromJson(Map<String, dynamic> json) => _$LineItemTaxFromJson(json);
  Map<String, dynamic> toJson() => _$LineItemTaxToJson(this);
}

@JsonSerializable()
class LineItemDiscount {
  @JsonValue('catalog_object_id')
  final String catalogObjectId;
  final String name;
  final String type;
  final String percentage;
  @JsonValue('amount_money')
  final Money amountMoney;
  @JsonValue('applied_money')
  final Money appliedMoney;
  final String scope;

  LineItemDiscount(this.catalogObjectId, this.name, this.type, this.percentage, this.appliedMoney, this.amountMoney, this.scope);

  factory LineItemDiscount.fromJson(Map<String, dynamic> json) => _$LineItemDiscountFromJson(json);
  Map<String, dynamic> toJson() => _$LineItemDiscountToJson(this);
}

