import 'package:dart_square/shared.dart';
import 'package:json_annotation/json_annotation.dart';

part 'transaction.g.dart';

@JsonSerializable()
class Transaction {
  final String id;
  @JsonValue('location_id')
  final String locationId;
  @JsonValue("created_at")
  final String createdAt;
  final List<Tender> tenders;
  final List<Refund> refunds;
  @JsonValue('reference_id')
  final String referenceId;
  final String product;
  @JsonValue("client_id")
  final String clientId;
  @JsonValue("shipping_address")
  final Address shippingAddress;
  @JsonValue("order_id")
  final String orderId;


  Transaction({ this.id, this.locationId, this.referenceId, 
    this.tenders, this.createdAt, this.refunds, this.product, 
    this.clientId, this.orderId, this.shippingAddress });

  factory Transaction.fromJson(Map<String, dynamic> json) => _$TransactionFromJson(json);
  Map<String, dynamic> toJson() => _$TransactionToJson(this);
}

@JsonSerializable()
class Tender {
  String id;
  @JsonValue('location_id')
  String locationId;
  @JsonValue('transaction_id')
  String transactionId;
  @JsonValue('created_at')
  String createdAt;
  String note;
  @JsonValue('amount_money')
  Money amount;
  @JsonValue('tip_money')
  Money tip;
  @JsonValue('processing_fee_money')
  Money processingFee;
  @JsonValue('customer_id')
  String customerId;
  String type;
  @JsonValue('card_details')
  TenderCardDetails cardDetails;
  @JsonValue('cash_details')
  TenderCashDetails cashDetails;
  @JsonValue('additional_recipients')
  List<AdditionalRecipient> additionalRecipient;

  Tender({ this.id, this.locationId, this.transactionId, this.createdAt, 
    this.note, this.amount, this.tip, this.processingFee, 
    this.customerId, this.type, this.cardDetails, this.cashDetails, 
    this.additionalRecipient });

  factory Tender.fromJson(Map<String, dynamic> json) => _$TenderFromJson(json);
  Map<String, dynamic> toJson() => _$TenderToJson(this);
}

@JsonSerializable()
class TenderCardDetails {
  String status;
  Card card;
  @JsonValue('entry_method')
  String entryMethod;
  
  TenderCardDetails({this.status, this.card, this.entryMethod});

  factory TenderCardDetails.fromJson(Map<String, dynamic> json) => _$TenderCardDetailsFromJson(json);
  Map<String, dynamic> toJson() => _$TenderCardDetailsToJson(this);
}

@JsonSerializable()
class TenderCashDetails {
  @JsonValue('buyer_tendered_money')
  Money buyerTenderedMoney;
  @JsonValue('change_back_money')
  Money changeBackMoney;
  
  TenderCashDetails({ this.buyerTenderedMoney, this.changeBackMoney });

  factory TenderCashDetails.fromJson(Map<String, dynamic> json) => _$TenderCashDetailsFromJson(json);
  Map<String, dynamic> toJson() => _$TenderCashDetailsToJson(this);
}

@JsonSerializable()
class Refund {
  String id;
  @JsonValue('location_id')
  String locationId;
  @JsonValue('transaction_id')
  String transactionId;
  @JsonValue('tender_id')
  String tenderId;
  @JsonValue('created_at')
  String createdAt;
  String reason;
  @JsonValue('amount_money')
  Money amount;
  String status;
  @JsonValue('processing_fee_money')
  Money processingFee;
  @JsonValue('additional_recipients')
  List<AdditionalRecipient> additionalRecipients;

  Refund({this.id, this.locationId, this.transactionId, this.tenderId, 
    this.createdAt, this.reason, this.amount, this.status, this.processingFee, 
    this.additionalRecipients});

  factory Refund.fromJson(Map<String, dynamic> json) => _$RefundFromJson(json);
  Map<String, dynamic> toJson() => _$RefundToJson(this);
}
