import 'dart:async';
import 'dart:convert';
import 'package:http/http.dart';
import 'package:reflutter/reflutter.dart';
import 'responses/responses.dart';
import 'requests/requests.dart';

export 'responses/responses.dart';
export 'requests/requests.dart';

part 'transactions.api.dart';

@ReflutterHttp(name: 'SquareTransactions')
abstract class _SquareTransactionsEndpoint {

  @Post('/v2/locations/:locationId/transactions/:transactionId/capture')
  Future<ReflutterResponse<CaptureTransactionResponse>> capture(
    @Param("location_id") String locationId,
    @Param("transaction_id") String transactionId
  );

  @Post('/v2/locations/:locationId/transactions')
  Future<ReflutterResponse<ChargeTransactionResponse>> charge(
    @Param("location_id") String locationId,
    @Body() ChargeRequest req
  );

  @Post('/v2/locations/:locationId/transactions/:transactionId/refund')
  Future<ReflutterResponse<CreateRefundResponse>> createRefund(
    @Param('location_id') String locationId,
    @Param('transaction_id') String transactionId,
    @Body() RefundRequest req
  );

  @Get('/v2/locations/:locationId/refunds')
  Future<ReflutterResponse<ListRefundResponse>> listRefunds(
    @Param('location_id') String locationId,
    @QueryParam('begin_time') String beginTime,
    @QueryParam('end_time') String end_time,
    @QueryParam('sort_order') String sort_order,
    @QueryParam() String cursor
  );

  @Get('/v2/locations/:locationId/transactions')
  Future<ReflutterResponse<ListTransactionResponse>> listTransactions(
    @Param('location_id') String locationId,
    @QueryParam('begin_time') String beginTime,
    @QueryParam('end_time') String end_time,
    @QueryParam('sort_order') String sort_order,
    @QueryParam() String cursor
  );

  @Get('/v2/locations/:locationId/transactions/:transactionId')
  Future<ReflutterResponse<GetTransactionResponse>> getTransaction(
    @Param('location_id') String locationId,
    @Param('tranaction_id') String transactionId
  );

  @Post('/v2/locaitons/:locationId/transactions/:transactionId/void')
  Future<ReflutterResponse<VoidTransactionResponse>> voidTransaction(
    @Param('location_id') String locationId,
    @Param('tranaction_id') String transactionId
  );

}